package ru.dbotthepony.mc.otm.block.entity;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.client.multiplayer.ClientLevel;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.nbt.ByteTag;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.StringTag;
import net.minecraft.network.chat.Component;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.MenuProvider;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.capabilities.ICapabilityProvider;
import net.minecraftforge.common.util.LazyOptional;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import java.lang.ref.WeakReference;
import java.util.function.Consumer;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
@SuppressWarnings("unused")
public abstract class BlockEntityMattery extends BlockEntity implements MenuProvider {
	protected Component display_name;
	protected RedstoneSetting redstone_setting = RedstoneSetting.LOW;
	protected int redstone_signal = 0;

	public BlockEntityMattery(BlockEntityType<?> p_155228_, BlockPos p_155229_, BlockState p_155230_) {
		super(p_155228_, p_155229_, p_155230_);
	}

	public void setDisplayName(Component text) {
		display_name = text;
	}

	abstract protected Component getDefaultDisplayName();

	protected void tickOnce(Runnable func) {
		if (level != null)
			OverdriveThatMatters.tickOnce(level, func);
	}

	protected void tickOnceServer(Runnable func) {
		if (level instanceof ServerLevel level)
			OverdriveThatMatters.tickOnce(level, func);
	}

	protected void tickOnceClient(Runnable func) {
		if (level instanceof ClientLevel level)
			OverdriveThatMatters.tickOnce(level, func);
	}

	protected void tickOnce(Consumer<Level> func) {
		if (level != null)
			OverdriveThatMatters.tickOnceSelf(level, func);
	}

	protected void tickOnceServer(Consumer<Level> func) {
		if (level instanceof ServerLevel level)
			OverdriveThatMatters.tickOnceSelf(level, func);
	}

	protected void tickOnceClient(Consumer<Level> func) {
		if (level instanceof ClientLevel level)
			OverdriveThatMatters.tickOnceSelf(level, func);
	}

	protected <T> LazyOptional<T> getAndBind(LazyOptional<T> old, @Nullable ICapabilityProvider provider, Capability<T> cap, @Nullable Direction side, Runnable invalidate) {
		final var get = provider != null ? provider.getCapability(cap, side) : LazyOptional.empty();

		if (old != get) {
			if (get.isPresent()) {
				final var ref = new WeakReference<>(invalidate);

				get.addListener((l) -> {
					final var self = ref.get();

					if (self != null) {
						self.run();
					}
				});
			}

			return get.cast();
		}

		return get.cast();
	}

	@Override
	public Component getDisplayName() {
		return display_name != null ? display_name : getDefaultDisplayName();
	}

	@Nullable
	@Override
	abstract public AbstractContainerMenu createMenu(int containerID, Inventory inventory, Player ply);

	protected void redstoneStatusUpdated(boolean new_blocked, boolean old_blocked) {

	}

	public void setRedstoneSignal(int level) {
		final var old = isBlockedByRedstone();
		redstone_signal = level;
		final var state = isBlockedByRedstone();

		if (old != state) {
			redstoneStatusUpdated(state, old);
		}
	}

	public void setRedstoneSetting(RedstoneSetting setting) {
		final var old = isBlockedByRedstone();
		redstone_setting = setting;
		final var state = isBlockedByRedstone();

		if (old != state) {
			redstoneStatusUpdated(state, old);
		}
	}

	public RedstoneSetting getRedstoneSetting() {
		return redstone_setting;
	}

	protected boolean isBlockedByRedstone() {
		switch (redstone_setting) {
			case LOW -> {
				return redstone_signal > 0;
			}

			case HIGH -> {
				return redstone_signal <= 0;
			}
		}

		return false;
	}

	@Override
	public void saveAdditional(CompoundTag nbt) {
		super.saveAdditional(nbt);

		if (display_name != null)
			nbt.putString("Name", Component.Serializer.toJson(display_name));

		nbt.putByte("redstone", (byte) redstone_setting.ordinal());
		nbt.putByte("redstone_signal", (byte) redstone_signal);
	}

	public void load(CompoundTag nbt) {
		super.load(nbt);

		if (nbt.get("Name") instanceof StringTag tag)
			display_name = Component.Serializer.fromJson(tag.getAsString());

		if (nbt.get("redstone") instanceof ByteTag tag)
			redstone_setting = RedstoneSetting.get(tag.getAsByte());

		if (nbt.get("redstone_signal") instanceof ByteTag tag)
			redstone_signal = tag.getAsByte();
	}

	// Just to mark chunk unsaved
	public void setChangedLight() {
		if (level != null) {
			if (level.hasChunkAt(getBlockPos()))
				level.getChunkAt(getBlockPos()).setUnsaved(true);
		}
	}
}
