package ru.dbotthepony.mc.otm.client.screen;

import net.minecraft.network.chat.Component;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.item.ItemStack;
import ru.dbotthepony.mc.otm.client.screen.panels.*;
import ru.dbotthepony.mc.otm.menu.ItemMonitorMenu;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

public class ItemMonitorScreen extends MatteryScreen<ItemMonitorMenu> {
	public static final int FRAME_WIDTH = 210;
	public static final int FRAME_HEIGHT = 110;

	public static final int GRID_WIDTH = 9;
	public static final int GRID_HEIGHT = 5;

	public ItemMonitorScreen(ItemMonitorMenu menu, Inventory inventory, Component title) {
		super(menu, inventory, title);
	}

	@Nullable
	@Override
	protected FramePanel makeMainFrame() {
		var frame = new FramePanel(this, null, 0, 0, FRAME_WIDTH, FRAME_HEIGHT, getTitle());

		var scroll_bar = new ScrollBarPanel(this, frame, 0, 0, 0);
		scroll_bar.setDock(Dock.RIGHT);
		scroll_bar.setupRowMultiplier(() -> {
			return menu.view.getItems().size() / GRID_WIDTH;
		});

		var grid = new GridPanel(this, frame, 0, 0, GRID_WIDTH * 18, 0, GRID_WIDTH, GRID_HEIGHT);
		grid.setDock(Dock.RIGHT);
		grid.setDockMargin(2, 2, 2, 2);

		for (int i = 0; i < GRID_WIDTH * GRID_HEIGHT; i++) {
			final int index = i;

			new AbstractSlotPanel(this, grid, 0, 0) {
				@Nonnull
				@Override
				protected ItemStack getItemStack() {
					int findex = index + scroll_bar.getScroll(menu.view.getItems().size() / GRID_WIDTH);

					var list = menu.view.getItems();

					if (findex >= list.size()) {
						return ItemStack.EMPTY;
					}

					return list.get(findex).stack();
				}

				@Override
				protected boolean mouseScrolledInner(double mouse_x, double mouse_y, double scroll) {
					return scroll_bar.mouseScrolledInner(mouse_x, mouse_y, scroll);
				}

				@Override
				protected boolean mouseClickedInner(double mouse_x, double mouse_y, int mouse_click_type) {
					int findex = index + scroll_bar.getScroll(GRID_WIDTH);
					menu.view.mouseClick(findex, mouse_click_type);

					return true;
				}
			};
		}

		return frame;
	}
}
