package ru.dbotthepony.mc.otm.capability.android

import net.minecraft.ChatFormatting
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.ListTag
import net.minecraft.nbt.Tag
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.resources.ResourceLocation
import net.minecraft.server.level.ServerPlayer
import net.minecraft.world.entity.Entity
import net.minecraft.world.entity.player.Player
import net.minecraftforge.event.AttachCapabilitiesEvent
import net.minecraftforge.event.entity.player.PlayerEvent.Clone
import net.minecraftforge.event.entity.player.PlayerEvent.PlayerChangedDimensionEvent
import net.minecraftforge.eventbus.api.SubscribeEvent
import ru.dbotthepony.mc.otm.Registry
import ru.dbotthepony.mc.otm.android.AndroidResearch
import ru.dbotthepony.mc.otm.android.AndroidResearchType
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.core.Fraction
import ru.dbotthepony.mc.otm.network.android.AndroidResearchPacket
import ru.dbotthepony.mc.otm.network.android.AndroidStatusPacket
import ru.dbotthepony.mc.otm.set
import java.util.*

class AndroidCapabilityPlayer(@JvmField val ply: Player) : AndroidCapability(ply) {
	@JvmField
	var isAndroid = false
	private var isAndroidNetwork = false

	@JvmField
	var willBecomeAndroid = false
	private var willBecomeAndroidNetwork = false
	private val research = ArrayList<AndroidResearch>()

	override fun invalidateNetworkState() {
		super.invalidateNetworkState()

		isAndroidNetwork = false
		willBecomeAndroidNetwork = false

		for (instance in research) {
			instance.dirty = true
		}
	}

	override fun isAndroid(): Boolean = isAndroid
	fun isEverAndroid(): Boolean = isAndroid || willBecomeAndroid

	fun becomeAndroidSoft() {
		if (isAndroid || willBecomeAndroid) return
		willBecomeAndroid = true
		(ply as? ServerPlayer)?.displayClientMessage(TranslatableComponent("otm.pill.message").withStyle(ChatFormatting.GRAY), false)
	}

	fun becomeAndroid() {
		if (isAndroid) return

		isAndroid = true
		willBecomeAndroid = false
		battery = Fraction(60000)
		maxBattery = Fraction(60000)
	}

	fun becomeAndroidAndKill() {
		if (isAndroid) return

		becomeAndroid()
		ply.hurt(Registry.DAMAGE_BECOME_ANDROID, ply.maxHealth * 2)
	}

	fun becomeHumane() {
		if (willBecomeAndroid) willBecomeAndroid = false
		if (!isAndroid) return

		isAndroid = false
		battery = Fraction(0)
		maxBattery = Fraction(60000)
		dropBattery()
	}

	fun becomeHumaneAndKill() {
		if (willBecomeAndroid) willBecomeAndroid = false
		if (!isAndroid) return

		becomeHumane()
		ply.hurt(Registry.DAMAGE_BECOME_HUMANE, ply.maxHealth * 2)
	}

	override fun deserializeNBT(compound: CompoundTag?) {
		super.deserializeNBT(compound!!)

		isAndroid = compound.getBoolean("is_android")
		willBecomeAndroid = compound.getBoolean("will_become_android")
		research.clear()

		val list = compound.getList("research", Tag.TAG_COMPOUND.toInt())

		for (tag in list) {
			if (tag is CompoundTag) {
				val research = Registry.ANDROID_RESEARCH.getValue(ResourceLocation(tag.getString("id")))

				if (research != null) {
					val instance = research.factory(this)
					instance.deserializeNBT(tag)
					this.research.add(instance)
				}
			}
		}
	}

	override fun serializeNBT(): CompoundTag {
		val tag = super.serializeNBT()

		tag["is_android"] = isAndroid
		tag["will_become_android"] = willBecomeAndroid

		val list = ListTag()

		for (instance in research) {
			val researchTag = CompoundTag()
			instance.serializeNBT(researchTag)

			researchTag["id"] = instance.type.registryName!!.toString()
			list.add(researchTag)
		}

		tag["research"] = list
		return tag
	}

	fun <T : AndroidResearch> getResearch(type: AndroidResearchType<T>): T {
		for (instance in research) {
			if (instance.type === type) {
				return instance as T
			}
		}

		val instance = type.factory(this)
		research.add(instance)
		return instance
	}

	@JvmField
	var last_jump_ticks = 14

	override fun tickNetwork() {
		super.tickNetwork()
		if (isAndroid != isAndroidNetwork) {
			isAndroidNetwork = isAndroid
			sendNetwork(AndroidStatusPacket(AndroidStatusPacket.Type.IS_ANDROID, isAndroid))
		}

		if (willBecomeAndroid != willBecomeAndroidNetwork) {
			willBecomeAndroidNetwork = willBecomeAndroid
			sendNetwork(AndroidStatusPacket(AndroidStatusPacket.Type.WILL_BECOME_ANDROID, willBecomeAndroid))
		}

		for (instance in research) {
			if (instance.dirty) {
				instance.dirty = false
				sendNetwork(AndroidResearchPacket(instance))
			}
		}
	}

	@JvmField
	var sleep_ticks = 0

	override fun tickInnerClientAlways() {
		super.tickInnerClientAlways()

		if (willBecomeAndroid) {
			if (ply.isSleeping) {
				sleep_ticks++
			} else {
				sleep_ticks = 0
			}
		}
	}

	override fun tickServerAlways() {
		super.tickServerAlways()

		if (willBecomeAndroid) {
			if (ply.isSleeping) {
				sleep_ticks++

				if (sleep_ticks > SLEEP_TICKS_LIMIT) {
					becomeAndroid()
					sleep_ticks = 0

					(ply as? ServerPlayer)?.displayClientMessage(TranslatableComponent("otm.pill.message_finish").withStyle(ChatFormatting.DARK_RED), false)
				}
			} else {
				sleep_ticks = 0
			}
		}
	}

	public override fun tickServer() {
		super.tickServer()

		// TODO: Maybe passive drain?
		// extractEnergyInner(BigDecimal.valueOf(new Random().nextDouble()), false);
		if (ply.isSwimming && !hasFeature(Registry.AndroidFeatures.AIR_BAGS)) {
			ply.isSwimming = false
		}

		val stats = ply.foodData

		while (stats.foodLevel < 18 && extractEnergyInner(ENERGY_FOR_HUNGER_POINT, true) >= ENERGY_FOR_HUNGER_POINT) {
			extractEnergyInner(ENERGY_FOR_HUNGER_POINT, false)
			stats.foodLevel = stats.foodLevel + 1
		}

		// "block" quick regeneration
		// also cause power to generate while in peaceful
		while (stats.foodLevel > 18 && receiveEnergyInner(ENERGY_FOR_HUNGER_POINT, true) >= ENERGY_FOR_HUNGER_POINT) {
			receiveEnergyInner(ENERGY_FOR_HUNGER_POINT, false)
			stats.foodLevel = stats.foodLevel - 1
		}

		val foodLevel = stats.foodLevel.toFloat()

		if (stats.saturationLevel < foodLevel) {
			val extracted = extractEnergyInner(ENERGY_FOR_HUNGER_POINT * (foodLevel - stats.saturationLevel), false)
			stats.setSaturation(stats.saturationLevel + (extracted / ENERGY_FOR_HUNGER_POINT).toFloat())
		}

		if (stats.exhaustionLevel > 0f) {
			val extracted = extractEnergyInner(ENERGY_FOR_HUNGER_POINT * (stats.exhaustionLevel / 4f), false)
			stats.setExhaustion(stats.exhaustionLevel - (extracted / ENERGY_FOR_HUNGER_POINT).toFloat() * 4f)
		}
	}

	@Suppress("unused")
	companion object {
		@SubscribeEvent
		fun onAttachCapabilityEvent(event: AttachCapabilitiesEvent<Entity?>) {
			val ent = event.`object`

			if (ent is Player) {
				event.addCapability(Registry.Names.ANDROID_CAPABILITY, AndroidCapabilityPlayer(ent))
			}
		}

		@SubscribeEvent
		fun onPlayerChangeDimensionEvent(event: PlayerChangedDimensionEvent) {
			event.player.getCapability(MatteryCapability.ANDROID)
				.ifPresent { it.invalidateNetworkState() }
		}

		@SubscribeEvent
		fun onPlayerCloneEvent(event: Clone) {
			event.player.getCapability(MatteryCapability.ANDROID).ifPresent {
				var resolver = event.original.getCapability(MatteryCapability.ANDROID)

				if (!resolver.isPresent || resolver.resolve().isEmpty) {
					event.original.reviveCaps()
					resolver = event.original.getCapability(MatteryCapability.ANDROID)
				}

				if (!resolver.isPresent || resolver.resolve().isEmpty) {
					event.original.invalidateCaps()
					return@ifPresent
				}

				val original = resolver.resolve().get() as AndroidCapabilityPlayer

				if (original.willBecomeAndroid && event.isWasDeath) {
					original.becomeAndroid()
					(event.player as? ServerPlayer)?.displayClientMessage(TranslatableComponent("otm.pill.message_finish").withStyle(ChatFormatting.DARK_RED), false)
				}

				it.deserializeNBT(original.serializeNBT())
				it.invalidateNetworkState()
				event.original.invalidateCaps()
			}
		}

		val ENERGY_FOR_HUNGER_POINT = Fraction(1000)
		const val SLEEP_TICKS_LIMIT = 80
	}
}