package ru.dbotthepony.mc.otm.client.render

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.resources.ResourceLocation
import ru.dbotthepony.mc.otm.client.screen.UVWindingOrder

@JvmRecord
data class SkinElement(
	val texture: ResourceLocation = RenderHelper.WIDGETS,
	val image_x: Float,
	val image_y: Float,
	val rect_w: Float,
	val rect_h: Float,
	val defined_width: Float = 256f,
	val defined_height: Float = 256f
) {
	@JvmOverloads
	fun render(
		stack: PoseStack,
		x: Float = 0f,
		y: Float = 0f,
		width: Float = rect_w,
		height: Float = rect_h,
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1
	) {
		RenderSystem.setShaderTexture(0, texture)
		RenderSystem.enableBlend()
		RenderSystem.defaultBlendFunc()

		renderRaw(stack, x, y, width, height, winding)
	}

	@JvmOverloads
	fun renderPartial(
		stack: PoseStack,
		x: Float = 0f,
		y: Float = 0f,
		width: Float = rect_w,
		height: Float = rect_h,
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1
	) {
		RenderSystem.setShaderTexture(0, texture)
		RenderSystem.enableBlend()
		RenderSystem.defaultBlendFunc()

		renderRawPartial(stack, x, y, width, height, winding)
	}

	@JvmOverloads
	fun renderW(
		stack: PoseStack,
		x: Float = 0f,
		y: Float = 0f,
		width: Float = rect_w,
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1,
	) {
		render(stack, x, y, width = width, winding = winding)
	}

	@JvmOverloads
	fun renderH(
		stack: PoseStack,
		x: Float = 0f,
		y: Float = 0f,
		height: Float = rect_h,
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1,
	) {
		render(stack, x, y, height = height, winding = winding)
	}

	@JvmOverloads
	fun renderRaw(
		stack: PoseStack,
		x: Float = 0f,
		y: Float = 0f,
		width: Float = rect_w,
		height: Float = rect_h,
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1
	) {
		val u0 = image_x / defined_width
		val v0 = image_y / defined_height
		val u1 = (image_x + rect_w) / defined_width
		val v1 = (image_y + rect_h) / defined_height

		val winded = winding.translate(u0, v0, u1, v1)

		RenderHelper.drawTexturedRectUV(
			stack,
			x,
			y,
			width,
			height,
			winded.u0,
			winded.v0,
			winded.u1,
			winded.v1,
		)
	}

	@JvmOverloads
	fun renderRawPartial(
		stack: PoseStack,
		x: Float = 0f,
		y: Float = 0f,
		width: Float = rect_w,
		height: Float = rect_h,
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1
	) {
		val u0 = image_x / defined_width
		val v0 = image_y / defined_height
		val u1 = (image_x + width) / defined_width
		val v1 = (image_y + height) / defined_height

		val winded = winding.translate(u0, v0, u1, v1)

		RenderHelper.drawTexturedRectUV(
			stack,
			x,
			y,
			width,
			height,
			winded.u0,
			winded.v0,
			winded.u1,
			winded.v1,
		)
	}
}
